@@ -22,6 +22,7 @@ from jax import linear_util as lu
 from jax.interpreters import ad
 from jax.interpreters import batching
 from jax.interpreters import partial_eval as pe
+from jax.interpreters import mlir
 from jax.interpreters import xla
 from jax.tree_util import tree_flatten, tree_unflatten
 from jax._src import ad_util
@@ -276,10 +277,12 @@ def remat_impl(*args, jaxpr, prevent_cse, differentiated, policy):
   del prevent_cse, differentiated, policy  # Unused.
   return core.eval_jaxpr(jaxpr, (), *args)
 
-@remat_p.def_abstract_eval
+@remat_p.def_effectful_abstract_eval
 def remat_abstract_eval(*args, jaxpr, prevent_cse, differentiated, policy):
   del args, prevent_cse, differentiated, policy  # Unused.
-  return [v.aval for v in jaxpr.outvars]
+  if jaxpr.effects:
+    raise NotImplementedError('Effects not supported in `remat`.')
+  return [v.aval for v in jaxpr.outvars], jaxpr.effects
 
 def remat_jvp(primals, tangents, jaxpr, prevent_cse, differentiated, policy):
   assert not jaxpr.constvars
@@ -392,6 +395,7 @@ ad.primitive_jvps[name_p] = name_jvp
 
 xla.register_translation(name_p,
                          lambda ctx, avals_in, avals_out, x, *, name: [x])
+mlir.register_lowering(name_p, lambda ctx, x, *, name: [x])
 
 def name_batcher(args, dims, *, name):
   (x,), (d,) = args, dims
