@@ -329,12 +329,14 @@ def _custom_jvp_call_jaxpr_impl(*args, fun_jaxpr: core.ClosedJaxpr, **params):
 
 def _custom_jvp_call_jaxpr_abstract_eval(*args, fun_jaxpr: core.ClosedJaxpr, **params):
   del args, params
-  return fun_jaxpr.out_avals
+  if fun_jaxpr.effects:
+    raise NotImplementedError('Effects not supported in `custom_jvp`.')
+  return fun_jaxpr.out_avals, fun_jaxpr.effects
 
 custom_jvp_call_jaxpr_p = core.AxisPrimitive('custom_jvp_call_jaxpr')
 custom_jvp_call_jaxpr_p.multiple_results = True
 custom_jvp_call_jaxpr_p.def_impl(_custom_jvp_call_jaxpr_impl)
-custom_jvp_call_jaxpr_p.def_abstract_eval(_custom_jvp_call_jaxpr_abstract_eval)
+custom_jvp_call_jaxpr_p.def_effectful_abstract_eval(_custom_jvp_call_jaxpr_abstract_eval)
 CustomJVPCallPrimitive.initial_style = custom_jvp_call_jaxpr_p
 
 mlir.register_lowering(custom_jvp_call_jaxpr_p, mlir.lower_fun(
@@ -395,11 +397,11 @@ def _custom_jvp_call_jaxpr_vmap(
   return batched_outs, out_dims
 batching.axis_primitive_batchers[custom_jvp_call_jaxpr_p] = _custom_jvp_call_jaxpr_vmap
 
+xla.register_initial_style_primitive(custom_jvp_call_jaxpr_p)
 xla.register_translation(
     custom_jvp_call_jaxpr_p,
     xla.lower_fun(_custom_jvp_call_jaxpr_impl, new_style=True,
-                  multiple_results=True),
-    initial_style=True)
+                  multiple_results=True))
 
 # If a (multi)linear function is defined with a custom jvp, then
 # custom_jvp_call_jaxpr can appear in jaxprs to be transposed. Since it's
@@ -694,12 +696,14 @@ def _custom_vjp_call_jaxpr_impl(*args, fun_jaxpr, **_):
   return core.jaxpr_as_fun(fun_jaxpr)(*args)
 
 def _custom_vjp_call_jaxpr_abstract_eval(*_, fun_jaxpr, **__):
-  return fun_jaxpr.out_avals
+  if fun_jaxpr.effects:
+    raise NotImplementedError('Effects not supported in `custom_vjp`.')
+  return fun_jaxpr.out_avals, fun_jaxpr.effects
 
 custom_vjp_call_jaxpr_p = core.AxisPrimitive('custom_vjp_call_jaxpr')
 custom_vjp_call_jaxpr_p.multiple_results = True
 custom_vjp_call_jaxpr_p.def_impl(_custom_vjp_call_jaxpr_impl)
-custom_vjp_call_jaxpr_p.def_abstract_eval(_custom_vjp_call_jaxpr_abstract_eval)
+custom_vjp_call_jaxpr_p.def_effectful_abstract_eval(_custom_vjp_call_jaxpr_abstract_eval)
 CustomVJPCallPrimitive.initial_style = custom_vjp_call_jaxpr_p
 
 mlir.register_lowering(custom_vjp_call_jaxpr_p, mlir.lower_fun(
@@ -764,11 +768,11 @@ def _custom_vjp_call_jaxpr_vmap(
   return batched_outs, out_dims
 batching.axis_primitive_batchers[custom_vjp_call_jaxpr_p] = _custom_vjp_call_jaxpr_vmap
 
+xla.register_initial_style_primitive(custom_vjp_call_jaxpr_p)
 xla.register_translation(
     custom_vjp_call_jaxpr_p,
     xla.lower_fun(_custom_vjp_call_jaxpr_impl, new_style=True,
-                  multiple_results=True),
-    initial_style=True)
+                  multiple_results=True))
 
 batching.primitive_batchers[ad.custom_lin_p] = ad._raise_custom_vjp_error_on_jvp
 xla.register_translation(ad.custom_lin_p, ad._raise_custom_vjp_error_on_jvp)
@@ -1160,10 +1164,10 @@ linear_call_p.multiple_results = True
 linear_call_p.def_impl(_linear_call_impl)
 linear_call_p.def_abstract_eval(_linear_call_abstract_eval)
 ad.primitive_transposes[linear_call_p] = _linear_call_transpose_rule
+xla.register_initial_style_primitive(linear_call_p)
 xla.register_translation(linear_call_p,
                          xla.lower_fun(_linear_call_impl, new_style=True,
-                                       multiple_results=True),
-                         initial_style=True)
+                                       multiple_results=True))
 mlir.register_lowering(linear_call_p, mlir.lower_fun(
     _linear_call_impl, multiple_results=True))
 
@@ -1184,6 +1188,7 @@ unreachable_p.def_impl(unreachable_impl)
 # that errs. Since translation over-approximates concrete evaluation,
 # we err on translation for the time being.
 xla.register_translation(unreachable_p, unreachable_impl)
+mlir.register_lowering(unreachable_p, unreachable_impl)
 
 # Abstract evaluation proceeds without issue, to allow for staging
 unreachable_p.def_abstract_eval(lambda *_, out_avals, **__: out_avals)
