@@ -149,24 +149,45 @@ def _xla_call_impl(fun: lu.WrappedFun, *args, device, backend, name,
   compiled_fun = _xla_callable(fun, device, backend, name, donated_invars,
                                *arg_specs)
   try:
-    out = compiled_fun(*args)
+    return compiled_fun(*args)
   except FloatingPointError:
     assert config.jax_debug_nans or config.jax_debug_infs  # compiled_fun can only raise in this case
-    print("Invalid value encountered in the output of a jit/pmap-ed function. "
+    print("Invalid value encountered in the output of a jit-decorated function. "
           "Calling the de-optimized version.")
     # We want to run the wrapped function again (after _xla_callable already ran
     # it), but linear_util.WrappedFun instances are meant to be run only once.
     # In addition to re-executing the Python code, which is usually undesirable
-    # but which config.jax_debug_nans is meant to opt into, we'll be re-executing
-    # any linear_util.py-style side effects, i.e. re-populating Stores created
-    # by any transformation_with_aux's applied to fun. Since this is
-    # intentional here, to avoid "Store occupied" errors we clone the WrappedFun
-    # with empty stores.
+    # but which config.jax_debug_nans is meant to opt into, we'll be
+    # re-executing any linear_util.py-style side effects, i.e. re-populating
+    # Stores created by any transformation_with_aux's applied to fun. Since this
+    # is intentional here, to avoid "Store occupied" errors we clone the
+    # WrappedFun with empty stores.
     stores = [lu.Store() for _ in fun.stores]
     clone = lu.WrappedFun(fun.f, fun.transforms, stores, fun.params, fun.in_type)
+
     with core.new_sublevel():
-      _ = clone.call_wrapped(*args)  # probably won't return
-  return out
+      _ = clone.call_wrapped(*args)  # may raise, not return
+
+    # If control reaches this line, we got a NaN on the output of `compiled_fun`
+    # but not `clone.call_wrapped` on the same arguments. Let's tell the user.
+    fun_info = pe.fun_sourceinfo(fun.f)
+    msg = ("An invalid value was encountered in the output of the "
+           f"`jit`-decorated function {fun_info}. Because "
+           "config.jax_debug_nans and/or config.jax_debug_infs is set, the "
+           "de-optimized function (i.e., the function as if the `jit` "
+           "decorator were removed) was called in an attempt to get a more "
+           "precise error message. However, the de-optimized function did not "
+           "produce invalid values during its execution. This behavior can "
+           "result from `jit` optimizations causing the invalud value to be "
+           "produced. It may also arise from having nan/inf constants as "
+           "outputs, like `jax.jit(lambda ...: jax.numpy.nan)(...)`. "
+           "\n\n"
+           "It may be possible to avoid the invalid value by removing the "
+           "`jit` decorator, at the cost of losing optimizations. "
+           "\n\n"
+           "If you see this error, consider opening a bug report at "
+           "https://github.com/google/jax.")
+    raise FloatingPointError(msg)
 
 xla.xla_call_p.def_impl(_xla_call_impl)
 
@@ -206,6 +227,8 @@ def lower_xla_callable(fun: lu.WrappedFun, device, backend, name,
                         "for jit in {elapsed_time} sec"):
     jaxpr, out_avals, consts = pe.trace_to_jaxpr_final(
         fun, abstract_args, pe.debug_info_final(fun, "jit"), which_explicit)
+  if jaxpr.effects:
+    raise NotImplementedError('Lowering jaxprs with effects not supported.')
   if any(isinstance(c, core.Tracer) for c in consts):
     raise UnexpectedTracerError("Encountered an unexpected tracer.")
   # TODO(mattjj): handle argument pruning w/ dynamic shapes
@@ -226,7 +249,7 @@ def lower_xla_callable(fun: lu.WrappedFun, device, backend, name,
   backend = xb.get_device_backend(device) if device else xb.get_backend(backend)
 
   if (config.jax_dynamic_shapes and jaxpr_has_bints(jaxpr) and
-      backend.platform != 'iree'):
+      not _backend_supports_unbounded_dynamic_shapes(backend)):
     jaxpr, consts = pe.pad_jaxpr(jaxpr, consts)
 
   # Computations that only produce constants and/or only rearrange their inputs,
@@ -279,6 +302,10 @@ def lower_xla_callable(fun: lu.WrappedFun, device, backend, name,
       in_avals=abstract_args, out_avals=out_avals, kept_var_idx=kept_var_idx)
 
 
+def _backend_supports_unbounded_dynamic_shapes(backend: Backend) -> bool:
+  return backend.platform == 'iree'
+
+
 def prefetch(x):
   if isinstance(x, device_array.DeviceArray):
     x.copy_to_host_async()
@@ -406,12 +433,14 @@ num_buffers_handlers[core.ShapedArray] = lambda _: 1
 num_buffers_handlers[core.ConcreteArray] = lambda _: 1
 
 
-def _input_handler(which_explicit: Optional[Sequence[bool]],
+def _input_handler(backend: Backend,
+                   which_explicit: Optional[Sequence[bool]],
                    in_avals: Sequence[core.AbstractValue]
                    ) -> Optional[Callable]:
   # Extract implicit inputs, and pad bounded-size inputs to their max size.
   needs_implicit = which_explicit and not all(which_explicit)
-  needs_padding = any(type(in_avals[d.val]) is core.AbstractBInt  # type: ignore
+  needs_padding = any(backend.platform != 'iree' and
+                      type(in_avals[d.val]) is core.AbstractBInt  # type: ignore
                       for a in in_avals if type(a) is core.DShapedArray
                       for d in a.shape if type(d) is pe.DBIdx)
 
@@ -446,13 +475,16 @@ def _input_handler(which_explicit: Optional[Sequence[bool]],
     explicit_args_ = iter(explicit_args)
     args = [next(explicit_args_) if ex else None for ex in which_explicit]
     assert next(explicit_args_, None) is None
+    assert needs_implicit
     for i, j, k in implicit_args_from_axes:
       if args[i] is None:
         args[i] = args[j].shape[k]  # type: ignore
       else:
         if args[i] != args[j].shape[k]:
           raise Exception("inconsistent argument axis sizes for type")
-    return tuple([pad(x) if pad else x for x, pad in zip(args, padders)])
+    if needs_padding:
+      args = tuple([pad(x) if pad else x for x, pad in zip(args, padders)])
+    return args
   return elaborate_and_pad
 
 def _pad_arg(shape, x):
@@ -684,7 +716,7 @@ class XlaCompiledComputation(stages.Executable):
       out_avals: Sequence[core.AbstractValue],
       kept_var_idx: Set[int]) -> XlaCompiledComputation:
     sticky_device = device
-    input_handler = _input_handler(explicit_args, in_avals)
+    input_handler = _input_handler(backend, explicit_args, in_avals)
     result_handlers = map(partial(aval_to_result_handler, sticky_device),
                           out_avals)
     options = xb.get_compile_options(
