@@ -348,6 +348,13 @@ cholesky_p = standard_unop(_float | _complex, 'cholesky')
 ad.primitive_jvps[cholesky_p] = cholesky_jvp_rule
 batching.primitive_batchers[cholesky_p] = cholesky_batching_rule
 
+def _cholesky_lowering(ctx, x):
+  aval, = ctx.avals_out
+  return mhlo.CholeskyOp(mlir.aval_to_ir_type(aval), x,
+                         lower=ir.BoolAttr.get(True)).results
+
+mlir.register_lowering(cholesky_p, _cholesky_lowering)
+
 def _cholesky_cpu_gpu_translation_rule(potrf_impl, ctx, avals_in, avals_out,
                                        operand):
   operand_aval, = avals_in
@@ -787,6 +794,24 @@ ad.primitive_transposes[triangular_solve_p] = triangular_solve_transpose_rule
 batching.primitive_batchers[triangular_solve_p] = triangular_solve_batching_rule
 
 
+def _triangular_solve_lowering(
+    ctx, a, b, *, left_side, lower, transpose_a, conjugate_a, unit_diagonal):
+  out_aval, = ctx.avals_out
+  if conjugate_a and not transpose_a:
+    a = chlo.ConjOp(a)
+    conjugate_a = False
+  if not transpose_a:
+    transpose = "NO_TRANSPOSE"
+  else:
+    transpose = "ADJOINT" if conjugate_a else "TRANSPOSE"
+  return mhlo.TriangularSolveOp(
+      mlir.aval_to_ir_type(out_aval), a, b, ir.BoolAttr.get(left_side),
+      ir.BoolAttr.get(lower), ir.BoolAttr.get(unit_diagonal),
+      mhlo.TransposeAttr.get(transpose)).results
+
+mlir.register_lowering(triangular_solve_p, _triangular_solve_lowering)
+
+
 def _triangular_solve_cpu_translation_rule(
     ctx, avals_in, avals_out, a, b, *, left_side, lower, transpose_a,
     conjugate_a, unit_diagonal):
@@ -1477,15 +1502,29 @@ def svd_impl(operand, full_matrices, compute_uv):
   return xla.apply_primitive(svd_p, operand, full_matrices=full_matrices,
                              compute_uv=compute_uv)
 
+
+def _zeros_like_xla(c, aval):
+  zero = xops.Constant(c, np.array(0, aval.dtype))
+  return xops.Broadcast(zero, aval.shape)
+
+def _eye_like_xla(c, aval):
+  iota_shape = xla_client.Shape.array_shape(
+      xla.dtype_to_primitive_type(np.dtype(np.int32)), aval.shape)
+  x = xops.Eq(xops.Iota(c, iota_shape, len(aval.shape) - 1),
+              xops.Iota(c, iota_shape, len(aval.shape) - 2))
+  return xops.ConvertElementType(x, xla.dtype_to_primitive_type(aval.dtype))
+
 def _svd_translation_rule(ctx, avals_in, avals_out, operand, *, full_matrices,
                           compute_uv):
   operand_aval, = avals_in
   shape = operand_aval.shape
   m, n = shape[-2:]
   if m == 0 or n == 0:
-    return xla.lower_fun(_empty_svd, multiple_results=True, new_style=True)(
-      ctx, avals_in, avals_out, operand, full_matrices=full_matrices,
-        compute_uv=compute_uv)
+    out = [_zeros_like_xla(ctx.builder, avals_out[0])]
+    if compute_uv:
+      out.append(_eye_like_xla(ctx.builder, avals_out[1]))
+      out.append(_eye_like_xla(ctx.builder, avals_out[2]))
+    return out
 
   u, s, v = xops.SVD(operand)
   permutation = list(range(len(shape)))
@@ -1816,8 +1855,7 @@ def _schur_impl(operand, *, compute_schur_vectors, sort_eig_vals,
       select_callable=select_callable)
 
 
-def _schur_translation_rule(ctx, avals_in, avals_out, operand, *,
-                            compute_schur_vectors, sort_eig_vals):
+def _schur_translation_rule(ctx, *args, **kwargs):
   raise NotImplementedError(
       "Schur decomposition is only implemented on the CPU backend.")
 
@@ -1849,7 +1887,7 @@ def _schur_cpu_translation_rule(ctx, avals_in, avals_out, operand, *,
   _cpu_gees = lapack.gees
 
   if sort_eig_vals:
-    T, vs, sdim, info = _cpu_gees(
+    T, vs, _sdim, info = _cpu_gees(
         c,
         operand,
         jobvs=compute_schur_vectors,
@@ -1875,6 +1913,49 @@ def _schur_cpu_translation_rule(ctx, avals_in, avals_out, operand, *,
 
   return output
 
+def _schur_cpu_lowering(ctx, operand, *, compute_schur_vectors, sort_eig_vals,
+                        select_callable):
+  operand_aval, = ctx.avals_in
+  batch_dims = operand_aval.shape[:-2]
+
+  if sort_eig_vals:
+    T, vs, _sdim, info = lapack.gees_mhlo(
+        operand,
+        jobvs=compute_schur_vectors,
+        sort=sort_eig_vals,
+        select=select_callable)
+  else:
+    T, vs, info = lapack.gees_mhlo(
+        operand,
+        jobvs=compute_schur_vectors,
+        sort=sort_eig_vals,
+        select=select_callable)
+
+  ok = mlir.compare_mhlo(
+      info, mlir.full_like_aval(0, ShapedArray(batch_dims, np.dtype(np.int32))),
+      "EQ", "SIGNED")
+
+  T = _broadcasting_select_mhlo(
+      mhlo.BroadcastInDimOp(
+          ir.RankedTensorType.get(batch_dims + (1, 1),
+                                  ir.IntegerType.get_signless(1)),
+          ok,
+          mlir.dense_int_elements(range(len(batch_dims)))).result,
+      T, _nan_like_mhlo(ctx.avals_out[0]))
+  output = [T]
+  if compute_schur_vectors:
+    vs = _broadcasting_select_mhlo(
+        mhlo.BroadcastInDimOp(
+            ir.RankedTensorType.get(batch_dims + (1, 1),
+                                    ir.IntegerType.get_signless(1)),
+            ok,
+            mlir.dense_int_elements(range(len(batch_dims)))).result,
+        vs, _nan_like_mhlo(ctx.avals_out[1]))
+
+    output.append(vs)
+
+  return output
+
 
 def _schur_batching_rule(batched_args, batch_dims, *, compute_schur_vectors,
                          sort_eig_vals, select_callable):
@@ -1901,6 +1982,9 @@ schur_p.def_impl(_schur_impl)
 schur_p.def_abstract_eval(_schur_abstract_eval)
 xla.register_translation(schur_p, _schur_translation_rule)
 xla.register_translation(schur_p, _schur_cpu_translation_rule, platform='cpu')
+mlir.register_lowering(schur_p, _schur_translation_rule)
+if jax._src.lib.version >= (0, 3, 6):
+  mlir.register_lowering(schur_p, _schur_cpu_lowering, platform='cpu')
 batching.primitive_batchers[schur_p] = _schur_batching_rule
 ad.primitive_jvps[schur_p] = _schur_jvp_rule
 
