@@ -36,7 +36,7 @@ from jax._src import ad_util
 from jax._src.lax.lax import _device_put_raw
 from jax.interpreters import xla
 from jax._src.lib import xla_client
-from . import jax2tf as jax2tf_internal
+from jax.experimental.jax2tf import jax2tf as jax2tf_internal
 
 import numpy as np
 import tensorflow as tf  # type: ignore[import]
@@ -237,7 +237,7 @@ def _call_tf_abstract_eval(*_,
 call_tf_p.def_abstract_eval(_call_tf_abstract_eval)
 
 
-def _call_tf_translation_rule(builder: xla.XlaBuilder, *args_op,
+def _call_tf_translation_rule(ctx, avals_in, avals_out, *args_op,
                               function_flat_tf,
                               args_flat_sig_tf,
                               **_):
@@ -245,7 +245,7 @@ def _call_tf_translation_rule(builder: xla.XlaBuilder, *args_op,
   code_gen, _ = _code_generator_and_avals(function_flat_tf, args_flat_sig_tf,  # type: ignore
                                           code_gen_optional=False)
   assert code_gen is not None
-  return code_gen(builder, args_op)
+  return code_gen(ctx.builder, args_op)
 
 
 @functools.lru_cache(maxsize=128)
@@ -253,7 +253,8 @@ def _code_generator_and_avals(
     function_flat_tf,
     args_flat_sig_tf,
     code_gen_optional=False
-) -> Tuple[Optional[Callable[[xla.XlaBuilder, Sequence[xla.XlaOp]], xla.XlaOp]],
+) -> Tuple[Optional[Callable[[xla.XlaBuilder, Sequence[xla.XlaOp]],
+                             Sequence[xla.XlaOp]]],
            Sequence[core.ShapedArray]]:
   # Returns and caches a code generator (taking a builder and the
   # XlaOps for the arguments) and a sequence of result abstract shapes.
@@ -340,7 +341,7 @@ def _code_generator_and_avals(
           return None, result_avals
       msg = ("Error compiling TensorFlow function. call_tf can used " +
              "in a staged context (under jax.jit, lax.scan, etc.) only with " +
-             "compileable functions. " +
+             "compileable functions with static output shapes. " +
              "See https://github.com/google/jax/blob/main/jax/experimental/jax2tf/README.md#limitations-of-call-tf for a discussion.")
       raise ValueError(msg) from e
 
@@ -350,14 +351,14 @@ def _code_generator_and_avals(
   xla_comp_parameter_shapes = xla_comp.program_shape().parameter_shapes()
   found_parameter_avals = [
       core.ShapedArray(found_xla_shape.dimensions(),
-                       found_xla_shape.numpy_dtype())
+                       dtypes.canonicalize_dtype(found_xla_shape.numpy_dtype()))
       for found_xla_shape in xla_comp_parameter_shapes
   ]
   # Add the captured_inputs to args_flat_sig_tf
   expected_args_flat_sig_tf = list(args_flat_sig_tf) + list(captured_inputs)
   expected_parameter_avals = [
       core.ShapedArray(tuple(arg_sig.shape.as_list()),
-                       arg_sig.dtype.as_numpy_dtype)
+                       dtypes.canonicalize_dtype(arg_sig.dtype.as_numpy_dtype))
       for arg_sig in expected_args_flat_sig_tf]
   if found_parameter_avals != expected_parameter_avals:
     msg = ("Compiled TensorFlow function has unexpected parameter types " +
@@ -370,6 +371,14 @@ def _code_generator_and_avals(
 
   # Canonicalize the results; e.g., makes them x32 if JAX is in 32-bit mode
   def canonical_res_aval(res_shape: xla.XlaShape) -> core.ShapedArray:
+    if not res_shape.is_static():
+      msg = ("Compiled TensorFlow function has dynamic output shape " +
+             f"{res_shape}. call_tf can used " +
+             "in a staged context (under jax.jit, lax.scan, etc.) only with " +
+             "compileable functions with static output shapes. " +
+             "See https://github.com/google/jax/blob/main/jax/experimental/jax2tf/README.md#limitations-of-call-tf for a discussion.")
+      raise ValueError(msg)
+
     res_dtype = res_shape.numpy_dtype()
     jax_res_dtype = dtypes.canonicalize_dtype(res_dtype)
     return core.ShapedArray(res_shape.dimensions(), jax_res_dtype)
@@ -384,8 +393,9 @@ def _code_generator_and_avals(
 
   result_avals = tuple(map(canonical_res_aval, result_shapes))  # type: ignore
 
-  def code_gen(builder: xla.XlaBuilder, args_op: Sequence[xla.XlaOp]) -> xla.XlaOp:
-    captured_ops = [xops.ConstantLiteral(builder, np.asarray(inp))
+  def code_gen(builder: xla.XlaBuilder, args_op: Sequence[xla.XlaOp]
+              ) -> Sequence[xla.XlaOp]:
+    captured_ops = [xops.Constant(builder, np.asarray(inp))
                     for inp in captured_inputs]
 
     res_tf = xops.Call(builder, xla_comp, args_op + tuple(captured_ops))  # type: ignore
@@ -399,15 +409,14 @@ def _code_generator_and_avals(
             new_element_type=xla.dtype_to_primitive_type(res_aval.dtype))
       return res_op
 
-    results = [
+    return [
         post_process_result(i, res_aval, res_shape)
         for i, (res_aval, res_shape) in enumerate(zip(result_avals,
                                                       result_shapes))]
-    return xops.Tuple(builder, results)
 
   return code_gen, result_avals
 
-xla.translations[call_tf_p] = _call_tf_translation_rule
+xla.register_translation(call_tf_p, _call_tf_translation_rule)
 
 TfVal = jax2tf_internal.TfVal
 def _jax2tf_call_tf(*args: TfVal,
