@@ -100,7 +100,7 @@ class JaxPrimitiveTest(tf_test_util.JaxToTfTestCase):
   @primitive_harness.parameterized(
       primitive_harness.all_harnesses,
       include_jax_unimpl=False,
-      #one_containing="mode=GatherScatterMode.CLIP"
+      #one_containing="cumprod_dtype_by_fun_shape=float16[8,9]_axis=0_reverse=False"
   )
   @jtu.ignore_warning(
       category=UserWarning, message="Using reduced precision for gradient.*")
@@ -112,8 +112,10 @@ class JaxPrimitiveTest(tf_test_util.JaxToTfTestCase):
     func_jax = harness.dyn_fun
     args = harness.dyn_args_maker(self.rng())
     enable_xla = harness.params.get("enable_xla", True)
-    self.ConvertAndCompare(func_jax, *args, limitations=limitations,
-                           enable_xla=enable_xla)
+    associative_scan_reductions = harness.params.get("associative_scan_reductions", False)
+    with jax.jax2tf_associative_scan_reductions(associative_scan_reductions):
+      self.ConvertAndCompare(func_jax, *args, limitations=limitations,
+                             enable_xla=enable_xla)
 
   def test_primitive_coverage(self):
     """Fail if there are JAX primitives that are not implemented."""
