@@ -28,7 +28,6 @@ from jax._src.api import _check_callable, _check_arg
 from jax._src.config import config
 from jax._src import dispatch
 from jax._src import source_info_util
-from jax._src.lib import xla_extension_version
 from jax._src.api_util import (argnums_partial_except, flatten_axes,
                                flatten_fun_nokwargs, _ensure_index_tuple,
                                donation_vector, rebase_donate_argnums,
@@ -69,6 +68,26 @@ AUTO = pxla.AUTO
 _is_auto = pxla._is_auto
 
 
+class PjitCompiled(stages.Compiled):
+
+  @pxla.maybe_cached_property
+  def input_shardings(self) -> Sequence[pxla.PartitionSpec]:
+    return [pxla.array_mapping_to_axis_resources(i)
+            for i in self._executable._in_axes]  # pytype: disable=attribute-error
+
+  @pxla.maybe_cached_property
+  def output_shardings(self) -> Sequence[pxla.PartitionSpec]:
+    return [pxla.array_mapping_to_axis_resources(o)
+            for o in self._executable._out_axes]  # pytype: disable=attribute-error
+
+
+class PjitLowered(stages.Lowered):
+
+  def compile(self) -> PjitCompiled:
+    return PjitCompiled(self._lowering.compile(), self.args_info, self.out_tree,
+                        no_kwargs=self._no_kwargs)
+
+
 # TODO(yashkatariya): Add pjit microbenchmarks.
 def pjit(fun: Callable,
          in_axis_resources,
@@ -224,10 +243,6 @@ def pjit(fun: Callable,
       raise RuntimeError("pjit requires a non-empty mesh! Are you sure that "
                          "it's defined at the call site?")
 
-    if xla_extension_version < 60:
-      if any(d.platform not in {'gpu', 'tpu'} for d in mesh.devices.flat):
-        raise RuntimeError("pjit only supports GPU and TPU devices")
-
     f = lu.wrap_init(fun)
     f, dyn_args = argnums_partial_except(f, static_argnums, args, allow_invalid=False)
     del args
@@ -301,7 +316,7 @@ def pjit(fun: Callable,
 
     args_kwargs_in_tree = treedef_tuple([in_tree, tree_flatten({})[1]])
     local_in_avals = args_kwargs_in_tree.unflatten(flat_local_in_avals)
-    return stages.Lowered.from_flat_info(
+    return PjitLowered.from_flat_info(
         lowering, args_kwargs_in_tree, local_in_avals, donate_argnums, out_tree,
         no_kwargs=True)
 
@@ -665,9 +680,11 @@ def _pjit_lower(
 
 def _pjit_abstract_eval(*args, jaxpr, out_axis_resources, resource_env,
                         out_positional_semantics, **_):
+  if jaxpr.effects:
+    raise NotImplementedError('Effects not supported in `pjit`.')
   return global_to_local(out_positional_semantics, resource_env.physical_mesh,
-                         jaxpr.out_avals, out_axis_resources)
-pjit_p.def_abstract_eval(_pjit_abstract_eval)
+                         jaxpr.out_avals, out_axis_resources), jaxpr.effects
+pjit_p.def_effectful_abstract_eval(_pjit_abstract_eval)
 
 
 def _pjit_translation_rule(ctx, avals_in, avals_out, *in_nodes, name,
