@@ -23,10 +23,13 @@ import numpy as np
 from jax import core
 from jax import lax
 from jax.interpreters import ad
+from jax.interpreters import mlir
 from jax.interpreters import xla
 from jax.experimental.sparse._base import JAXSparse
 from jax.experimental.sparse.util import _coo_extract, _safe_asarray, CuSparseEfficiencyWarning
 from jax import tree_util
+import jax._src.lib
+from jax._src.lib.mlir.dialects import mhlo
 from jax._src.lib import sparse_apis
 from jax._src.numpy.lax_numpy import _promote_dtypes
 from jax._src.lib import xla_client
@@ -54,7 +57,8 @@ class COO(JAXSparse):
   nse = property(lambda self: self.data.size)
   dtype = property(lambda self: self.data.dtype)
   _info = property(lambda self: COOInfo(
-      shape=self.shape, rows_sorted=self._rows_sorted, cols_sorted=self._cols_sorted))
+      shape=self.shape, rows_sorted=self._rows_sorted,
+      cols_sorted=self._cols_sorted))
   _bufs = property(lambda self: (self.data, self.row, self.col))
   _rows_sorted: bool
   _cols_sorted: bool
@@ -69,16 +73,18 @@ class COO(JAXSparse):
   def fromdense(cls, mat, *, nse=None, index_dtype=np.int32):
     return coo_fromdense(mat, nse=nse, index_dtype=index_dtype)
 
-  def _sort_rows(self):
-    """Return a copy of the COO matrix with sorted rows.
+  def _sort_indices(self):
+    """Return a copy of the COO matrix with sorted indices.
 
+    The matrix is sorted by row indices and column indices per row.
     If self._rows_sorted is True, this returns ``self`` without a copy.
     """
     # TODO(jakevdp): would be benefit from lowering this to cusparse sort_rows utility?
     if self._rows_sorted:
       return self
-    row, col, data = lax.sort((self.row, self.col, self.data), num_keys=1)
-    return self.__class__((data, row, col), shape=self.shape, rows_sorted=True)
+    row, col, data = lax.sort((self.row, self.col, self.data), num_keys=2)
+    return self.__class__((data, row, col), shape=self.shape,
+                          rows_sorted=True)
 
   @classmethod
   def _empty(cls, shape, *, dtype=None, index_dtype='int32'):
@@ -88,7 +94,8 @@ class COO(JAXSparse):
       raise ValueError(f"COO must have ndim=2; got shape={shape}")
     data = jnp.empty(0, dtype)
     row = col = jnp.empty(0, index_dtype)
-    return cls((data, row, col), shape=shape, rows_sorted=True, cols_sorted=True)
+    return cls((data, row, col), shape=shape, rows_sorted=True,
+               cols_sorted=True)
 
   def todense(self):
     return coo_todense(self)
@@ -182,6 +189,36 @@ def _coo_todense_gpu_translation_rule(ctx, avals_in, avals_out, data, row, col,
 
 
   return [xops.Transpose(result, (1, 0))] if transpose else [result]
+_coo_todense_lowering = mlir.lower_fun(
+    _coo_todense_impl, multiple_results=False)
+
+def _coo_todense_gpu_lowering(ctx, data, row, col, *, spinfo):
+  data_aval, row_aval, _ = ctx.avals_in
+  dtype = data_aval.dtype
+  if not (np.issubdtype(dtype, np.floating) or np.issubdtype(dtype, np.complexfloating)):
+    warnings.warn(f"coo_todense cusparse/hipsparse lowering not available for dtype={dtype}. "
+                  "Falling back to default implementation.", CuSparseEfficiencyWarning)
+    return _coo_todense_lowering(ctx, data, row, col, spinfo=spinfo)
+
+  if spinfo.rows_sorted:
+    shape = spinfo.shape
+    transpose = False
+  elif spinfo.cols_sorted:
+    row, col = col, row
+    transpose = True
+    shape = spinfo.shape[::-1]
+  else:
+    warnings.warn("coo_todense GPU lowering requires matrices with sorted rows or sorted cols. "
+                  "To sort the rows in your matrix, use e.g. mat = mat._sort_rows(). Falling "
+                  "back to the default implementation.", CuSparseEfficiencyWarning)
+    return _coo_todense_lowering(ctx, data, row, col, spinfo=spinfo)
+
+  result = sparse_apis.coo_todense_mhlo(
+      data, row, col, shape=shape, data_dtype=dtype, index_dtype=row_aval.dtype)
+  return (
+      [mhlo.TransposeOp(result, mlir.dense_int_elements([1, 0])).result]
+      if transpose else [result])
+
 
 def _coo_todense_jvp(data_dot, data, row, col, *, spinfo):
   return _coo_todense(data_dot, row, col, spinfo=spinfo)
@@ -200,9 +237,13 @@ def _coo_todense_transpose(ct, data, row, col, *, spinfo):
 ad.defjvp(coo_todense_p, _coo_todense_jvp, None, None)
 ad.primitive_transposes[coo_todense_p] = _coo_todense_transpose
 xla.register_translation(coo_todense_p, _coo_todense_translation_rule)
+mlir.register_lowering(coo_todense_p, _coo_todense_lowering)
 if sparse_apis and sparse_apis.is_supported:
   xla.register_translation(coo_todense_p, _coo_todense_gpu_translation_rule,
                            platform='gpu')
+  if jax._src.lib.version > (0, 3, 5):
+    mlir.register_lowering(coo_todense_p, _coo_todense_gpu_lowering,
+                           platform='gpu')
 
 #--------------------------------------------------------------------
 # coo_fromdense
@@ -225,7 +266,8 @@ def coo_fromdense(mat, *, nse=None, index_dtype=jnp.int32):
   if nse is None:
     nse = (mat != 0).sum()
   nse = core.concrete_or_error(operator.index, nse, "coo_fromdense nse argument")
-  return COO(_coo_fromdense(mat, nse=nse, index_dtype=index_dtype), shape=mat.shape, rows_sorted=True)
+  return COO(_coo_fromdense(mat, nse=nse, index_dtype=index_dtype),
+             shape=mat.shape, rows_sorted=True)
 
 def _coo_fromdense(mat, *, nse, index_dtype=jnp.int32):
   """Create COO-format sparse matrix from a dense matrix.
@@ -278,6 +320,23 @@ def _coo_fromdense_gpu_translation_rule(ctx, avals_in, avals_out, mat, *, nse,
       ctx.builder, mat, nnz=nse, index_dtype=np.dtype(index_dtype))
   return [data, row, col]
 
+_coo_fromdense_lowering = mlir.lower_fun(
+    _coo_fromdense_impl, multiple_results=True)
+
+def _coo_fromdense_gpu_lowering(ctx, mat, *, nse, index_dtype):
+  dtype = ctx.avals_in[0].dtype
+  if not (np.issubdtype(dtype, np.floating) or np.issubdtype(dtype, np.complexfloating)):
+    warnings.warn(f"coo_fromdense cusparse/hipsparse lowering not available for dtype={dtype}. "
+                  "Falling back to default implementation.", CuSparseEfficiencyWarning)
+    return _coo_fromdense_lowering(ctx, mat, nse=nse, index_dtype=index_dtype)
+  data, row, col = sparse_apis.coo_fromdense_mhlo(
+      mat, nnz=nse,
+      data_dtype=dtype,
+      index_dtype=np.dtype(index_dtype),
+      index_type=mlir.dtype_to_ir_type(np.dtype(index_dtype)))
+  return [data, row, col]
+
+
 def _coo_fromdense_jvp(primals, tangents, *, nse, index_dtype):
   M, = primals
   Mdot, = tangents
@@ -307,10 +366,15 @@ ad.primitive_jvps[coo_fromdense_p] = _coo_fromdense_jvp
 ad.primitive_transposes[coo_fromdense_p] = _coo_fromdense_transpose
 
 xla.register_translation(coo_fromdense_p, _coo_fromdense_translation_rule)
+mlir.register_lowering(coo_fromdense_p, _coo_fromdense_lowering)
 if sparse_apis and sparse_apis.is_supported:
   xla.register_translation(coo_fromdense_p,
                            _coo_fromdense_gpu_translation_rule,
                            platform='gpu')
+  if jax._src.lib.version > (0, 3, 5):
+    mlir.register_lowering(coo_fromdense_p,
+                           _coo_fromdense_gpu_lowering,
+                           platform='gpu')
 
 #--------------------------------------------------------------------
 # coo_matvec
@@ -400,6 +464,36 @@ def _coo_matvec_gpu_translation_rule(ctx, avals_in, avals_out, data, row, col,
   return [sparse_apis.coo_matvec(ctx.builder, data, row, col, v, shape=shape,
                                  transpose=transpose)]
 
+_coo_matvec_lowering = mlir.lower_fun(
+    _coo_matvec_impl, multiple_results=False)
+
+def _coo_matvec_gpu_lowering(ctx, data, row, col, v, *, spinfo, transpose):
+  data_aval, row_aval, _, x_aval = ctx.avals_in
+  dtype = data_aval.dtype
+  if dtype not in [np.float32, np.float64, np.complex64, np.complex128]:
+    warnings.warn(f"coo_matvec cusparse/hipsparse lowering not available for dtype={dtype}. "
+                  "Falling back to default implementation.", CuSparseEfficiencyWarning)
+    return _coo_matvec_lowering(ctx, data, row, col, v, spinfo=spinfo,
+                                transpose=transpose)
+
+  if spinfo.rows_sorted:
+    shape = spinfo.shape
+  elif spinfo.cols_sorted:
+    row, col = col, row
+    transpose = not transpose
+    shape = spinfo.shape[::-1]
+  else:
+    warnings.warn("coo_matvec GPU lowering requires matrices with sorted rows or sorted cols. "
+                  "To sort the rows in your matrix, use e.g. mat = mat._sort_rows(). Falling "
+                  "back to the default implementation.", CuSparseEfficiencyWarning)
+    return _coo_matvec_lowering(ctx, data, row, col, v, spinfo=spinfo,
+                                transpose=transpose)
+
+  return [sparse_apis.coo_matvec_mhlo(
+      data, row, col, v, shape=shape, transpose=transpose,
+      index_dtype=row_aval.dtype, data_dtype=dtype, x_dtype=x_aval.dtype)]
+
+
 def _coo_matvec_jvp_mat(data_dot, data, row, col, v, *, spinfo, transpose):
   return _coo_matvec(data_dot, row, col, v, spinfo=spinfo, transpose=transpose)
 
@@ -421,9 +515,13 @@ def _coo_matvec_transpose(ct, data, row, col, v, *, spinfo, transpose):
 ad.defjvp(coo_matvec_p, _coo_matvec_jvp_mat, None, None, _coo_matvec_jvp_vec)
 ad.primitive_transposes[coo_matvec_p] = _coo_matvec_transpose
 xla.register_translation(coo_matvec_p, _coo_matvec_translation_rule)
+mlir.register_lowering(coo_matvec_p, _coo_matvec_lowering)
 if sparse_apis and sparse_apis.is_supported:
   xla.register_translation(coo_matvec_p, _coo_matvec_gpu_translation_rule,
                            platform='gpu')
+  if jax._src.lib.version > (0, 3, 5):
+    mlir.register_lowering(coo_matvec_p, _coo_matvec_gpu_lowering,
+                           platform='gpu')
 
 #--------------------------------------------------------------------
 # coo_matmat
@@ -495,6 +593,7 @@ def _coo_matmat_gpu_translation_rule(ctx, avals_in, avals_out, data, row, col,
                   "Falling back to default implementation.", CuSparseEfficiencyWarning)
     return _coo_matmat_translation_rule(ctx, avals_in, avals_out, data, row, col, B,
                                         spinfo=spinfo, transpose=transpose)
+
   if spinfo.rows_sorted:
     shape = spinfo.shape
   elif spinfo.cols_sorted:
@@ -511,6 +610,35 @@ def _coo_matmat_gpu_translation_rule(ctx, avals_in, avals_out, data, row, col,
   return [sparse_apis.coo_matmat(ctx.builder, data, row, col, B, shape=shape,
                                  transpose=transpose)]
 
+_coo_matmat_lowering = mlir.lower_fun(_coo_matmat_impl, multiple_results=False)
+
+def _coo_matmat_gpu_lowering(ctx, data, row, col, B, *, spinfo, transpose):
+  data_aval, row_aval, _, B_aval = ctx.avals_in
+  dtype = data_aval.dtype
+  if dtype not in [np.float32, np.float64, np.complex64, np.complex128]:
+    warnings.warn(f"coo_matmat cusparse/hipsprse lowering not available for dtype={dtype}. "
+                  "Falling back to default implementation.", CuSparseEfficiencyWarning)
+    return _coo_matmat_lowering(ctx, data, row, col, B, spinfo=spinfo,
+                                transpose=transpose)
+  if spinfo.rows_sorted:
+    shape = spinfo.shape
+  elif spinfo.cols_sorted:
+    row, col = col, row
+    transpose = not transpose
+    shape = spinfo.shape[::-1]
+  else:
+    warnings.warn("coo_matmat GPU lowering requires matrices with sorted rows or sorted cols. "
+                  "To sort the rows in your matrix, use e.g. mat = mat._sort_rows(). Falling "
+                  "back to the default implementation.", CuSparseEfficiencyWarning)
+    return _coo_matmat_lowering(ctx, data, row, col, B, spinfo=spinfo,
+                                transpose=transpose)
+
+  return [sparse_apis.coo_matmat_mhlo(data, row, col, B, shape=shape,
+                                      transpose=transpose, x_dtype=B_aval.dtype,
+                                      data_dtype=data_aval.dtype,
+                                      index_dtype=row_aval.dtype)]
+
+
 def _coo_matmat_jvp_left(data_dot, data, row, col, B, *, spinfo, transpose):
   return _coo_matmat(data_dot, row, col, B, spinfo=spinfo, transpose=transpose)
 
@@ -529,6 +657,10 @@ def _coo_matmat_transpose(ct, data, row, col, B, *, spinfo, transpose):
 ad.defjvp(coo_matmat_p, _coo_matmat_jvp_left, None, None, _coo_matmat_jvp_right)
 ad.primitive_transposes[coo_matmat_p] = _coo_matmat_transpose
 xla.register_translation(coo_matmat_p, _coo_matmat_translation_rule)
+mlir.register_lowering(coo_matmat_p, _coo_matmat_lowering)
 if sparse_apis and sparse_apis.is_supported:
   xla.register_translation(coo_matmat_p, _coo_matmat_gpu_translation_rule,
                            platform='gpu')
+  if jax._src.lib.version > (0, 3, 5):
+    mlir.register_lowering(coo_matmat_p, _coo_matmat_gpu_lowering,
+                           platform='gpu')
