@@ -55,7 +55,7 @@ import numpy as np
 from jax import core
 from jax import lax
 from jax import linear_util as lu
-from jax.experimental.sparse.bcoo import bcoo_multiply_dense, bcoo_multiply_sparse, BCOOInfo
+from jax.experimental.sparse.bcoo import bcoo_multiply_dense, bcoo_multiply_sparse
 import jax.numpy as jnp
 from jax._src.api_util import flatten_fun_nokwargs
 from jax.interpreters import partial_eval as pe
@@ -432,21 +432,11 @@ for _prim in [
   sparse_rules[_prim] = _zero_preserving_unary_op(_prim)
 
 def _dot_general_sparse(spenv, *spvalues, dimension_numbers, precision, preferred_element_type):
-  A, B = spvalues_to_arrays(spenv, spvalues)
-  if spvalues[0].is_sparse() and spvalues[1].is_sparse():
-    shape = sparse.bcoo._dot_general_validated_shape(A.shape, B.shape, dimension_numbers)
-    data, indices = sparse.bcoo_spdot_general(A.data, A.indices, B.data, B.indices,
-                                              lhs_spinfo=BCOOInfo(A.shape),
-                                              rhs_spinfo=BCOOInfo(B.shape),
-                                              dimension_numbers=dimension_numbers)
-    return [spenv.sparse(shape, data, indices)]
-  elif spvalues[0].is_sparse():
-    result = sparse.bcoo_dot_general(A.data, A.indices, B, lhs_spinfo=BCOOInfo(A.shape),
-                                     dimension_numbers=dimension_numbers)
-  else:
-    result = sparse.bcoo_rdot_general(A, B.data, B.indices, rhs_spinfo=BCOOInfo(B.shape),
-                                      dimension_numbers=dimension_numbers)
-  return [spenv.dense(result)]
+  # TODO(jakevdp): pass along these unused configurations?
+  del precision, preferred_element_type  # unused
+  result = sparse.bcoo_dot_general(*spvalues_to_arrays(spenv, spvalues),
+                                   dimension_numbers=dimension_numbers)
+  return arrays_to_spvalues(spenv, [result])
 
 sparse_rules[lax.dot_general_p] = _dot_general_sparse
 
@@ -454,9 +444,8 @@ def _transpose_sparse(spenv, *spvalues, permutation):
   permutation = tuple(permutation)
   args = spvalues_to_arrays(spenv, spvalues)
   shape = args[0].shape
-  data, indices = sparse.bcoo_transpose(args[0].data, args[0].indices,
-                                        permutation=permutation,
-                                        spinfo=BCOOInfo(shape))
+  mat = sparse.BCOO((args[0].data, args[0].indices), shape=shape)
+  mat_transposed = sparse.bcoo_transpose(mat, permutation=permutation)
   out_shape = tuple(shape[i] for i in permutation)
 
   n_batch = args[0].indices.ndim - 2
@@ -470,13 +459,13 @@ def _transpose_sparse(spenv, *spvalues, permutation):
   if batch_dims_unchanged and dense_dims_unchanged:
     kwds['data_ref'] = spvalues[0].data_ref
   else:
-    kwds['data'] = data
+    kwds['data'] = mat_transposed.data
 
   # Indices unchanged if batch & sparse dims are not permuted
   if batch_dims_unchanged and sparse_dims_unchanged:
     kwds['indices_ref'] = spvalues[0].indices_ref
   else:
-    kwds['indices'] = indices
+    kwds['indices'] = mat_transposed.indices
 
   spvalue = spenv.sparse(out_shape, **kwds)
   return (spvalue,)
@@ -518,16 +507,14 @@ def _mul_sparse(spenv, *spvalues):
       out_data = lax.mul(spenv.data(X), spenv.data(Y))
       out_spvalue = spenv.sparse(X.shape, out_data, indices_ref=X.indices_ref)
     else:
-      data, indices, shape = bcoo_multiply_sparse(spenv.data(X), spenv.indices(X),
-                                                  spenv.data(Y), spenv.indices(Y),
-                                                  lhs_spinfo=BCOOInfo(X.shape),
-                                                  rhs_spinfo=BCOOInfo(Y.shape))
-      out_spvalue = spenv.sparse(shape, data, indices)
+      X_promoted, Y_promoted = spvalues_to_arrays(spenv, spvalues)
+      mat = bcoo_multiply_sparse(X_promoted, Y_promoted)
+      out_spvalue = spenv.sparse(mat.shape, mat.data, mat.indices)
   else:
     if Y.is_sparse():
       X, Y = Y, X
-    out_data = bcoo_multiply_dense(
-        spenv.data(X), spenv.indices(X), spenv.data(Y), spinfo=BCOOInfo(X.shape))
+    X_promoted = spvalues_to_arrays(spenv, X)
+    out_data = bcoo_multiply_dense(X_promoted, spenv.data(Y))
     out_spvalue = spenv.sparse(X.shape, out_data, indices_ref=X.indices_ref)
 
   return (out_spvalue,)
@@ -536,22 +523,23 @@ sparse_rules[lax.mul_p] = _mul_sparse
 
 def _reduce_sum_sparse(spenv, *spvalues, axes):
   X, = spvalues
-  data, indices, out_shape = sparse.bcoo_reduce_sum(
-      spenv.data(X), spenv.indices(X), spinfo=BCOOInfo(X.shape), axes=axes)
+  X_promoted = spvalues_to_arrays(spenv, X)
+  mat = sparse.bcoo_reduce_sum(X_promoted, axes=axes)
+  out_shape = mat.shape
   if out_shape == ():
-    out_spvalue = spenv.dense(data.sum())
+    out_spvalue = spenv.dense(mat.data.sum())
   else:
-    out_spvalue = spenv.sparse(out_shape, data, indices)
+    out_spvalue = spenv.sparse(out_shape, mat.data, mat.indices)
   return (out_spvalue,)
 
 sparse_rules[lax.reduce_sum_p] = _reduce_sum_sparse
 
 def _broadcast_in_dim_sparse(spenv, *spvalues, shape, broadcast_dimensions):
   operand, = spvalues
-  data, indices = sparse.bcoo_broadcast_in_dim(spenv.data(operand), spenv.indices(operand),
-                                               spinfo=BCOOInfo(operand.shape), shape=shape,
-                                               broadcast_dimensions=broadcast_dimensions)
-  return (spenv.sparse(shape, data, indices),)
+  operand_promoted = spvalues_to_arrays(spenv, operand)
+  mat = sparse.bcoo_broadcast_in_dim(operand_promoted, shape=shape,
+                                     broadcast_dimensions=broadcast_dimensions)
+  return (spenv.sparse(shape, mat.data, mat.indices),)
 
 sparse_rules[lax.broadcast_in_dim_p] = _broadcast_in_dim_sparse
 
@@ -684,7 +672,7 @@ sparse_rules[lax.cond_p] = _cond_sparse
 
 def _todense_sparse_rule(spenv, spvalue, *, tree):
   del tree  # TODO(jakvdp): we should assert that tree is PytreeDef(*)
-  out = sparse.bcoo_todense(spenv.data(spvalue), spenv.indices(spvalue), spinfo=BCOOInfo(spvalue.shape))
+  out = sparse.BCOO((spenv.data(spvalue), spenv.indices(spvalue)), shape=spvalue.shape).todense()
   return (spenv.dense(out),)
 
 sparse_rules[sparse.todense_p] = _todense_sparse_rule
