@@ -492,7 +492,8 @@ class MaskTrace(Trace):
     if masking_rule is None:
       raise NotImplementedError(
         f'Masking rule for {primitive} not implemented yet.')
-    out_aval = primitive.abstract_eval(*(t.aval for t in tracers), **params)
+    # Ignore effects for now.
+    out_aval, _ = primitive.abstract_eval(*(t.aval for t in tracers), **params)
     vals, polymorphic_shapes = unzip2((t.val, t.polymorphic_shape) for t in tracers)
     logical_shapes = map(shape_as_value, polymorphic_shapes)
     # TODO(mattjj): generalize mask rule signature
